/*
================================================================================

ProdMe dashboard widget

Author: Jim Carlson (jacF93@hampshire.edu, http://ucsu.colorado.edu/~carlson)
  Date: 5/17/06

================================================================================
*/

// -----------------------------------------------------------------------------
// Globals
// -----------------------------------------------------------------------------
var gSoundNamesPaths = null;
var gPathToSound = "";		// Holds path to selected notification sound
var gVolume = 3;			// User setting: Volume of the alert
//var gLastTime = null;		// Date object for last time alert played
var gLastBGColor = 1;
var gChimeType = "off";     // User setting: Which type of chime has the user selected?
var g24HourClock = false;
var gDrawerShown = false;   // User setting: Is the "advanced" prefs drawer shown, or not?

// Text alert variables
var gTextMessage = "Very important reminder!";
var gShowTextMessage = false;   // User setting: Show text alert, or not?
var gUseGrowl = false;          // User setting: Use Growl if possible?
var gGrowlExists = false;       // This will be set to true if we can verify that Growl is present on the system

// Mute-related variables
var gIsMuted = false;   // User setting: This variable reflects the manually selected mute status, not taking into account the "mute interval" feature.
var gMuteBetweenEnabled = false;    // User setting: Is automatic mute enabled?
var gMuteBetweenStart = 0;          // User setting: The time when the mute is turned on
var gMuteBetweenEnd = 0;            // User setting: The time when the mute is turned off

// Default & current sizes for widget parts
var gDefaultFrontWidth  = 209;
var gDefaultFrontHeight = 158;
var gCurrentFrontWidth  = 209;
var gCurrentFrontHeight = 158;
var gDefaultBackWidth  = 231;   // No need for default height, that I can see
var gCurrentBackWidth  = 231;
var gCurrentBackHeight = 240;


// Clock chime variables
var gClockChimeInterval = 1;        // User setting: Number of minutes between chimes
var gClockChimeStrikeHour = false;  // User setting: Strike the hour?


// OS9 style chime variables
var gOS9Chimes = new Array;   // User setting(s): Specifies the sound for each of the four chimes
var gOS9StrikeHour = false;   // User setting: Strike the hour?


// Alarm clock & countdown timer variables
var gAlarmTime = 0;         // User setting: The time of the alarm
var gAlarmRepeatUntilDismissed = true;  // User setting: Does the alarm continue playing until the user dismisses it?
var gAlarmSnoozeLength = 9; // Number of minutes to snooze
var gAlarmSnoozeOn = false; // User setting: Is the snooze feature turned on?
var gAlarmOn = false;       // Is the alarm currently on (either playing or in snooze mode)?
var gAlarmMaxTime = 60;     // Maximum number of minutes to repeat alarm when user hasn't dismissed it yet
var gAlarmSnoozeStartTime = null;    // Set when the user presses the snooze button
var gAlarmStartTime = null; // Should set this when the alarm goes off (including snoozed alarms)


// "Other" chime variables
var gOtherPresetInterval = 1;   // User setting: Which interval has the user selected from the preset menu?
var gOtherCustomInterval = 1;   // User setting: Custom interval, in minutes
var gOtherUseCustomInterval = false; // User setting: Is a custom time interval set, or is a preset being used?


// Declarations to make jslint more useful
/*global alert, document, event, getElem, getUser, clearTimeout, setTimeout, calculateAndShowThumb, clearPref, disableButton, enableButton, get12HourTime, getWidgetRoot, is24HourClock, retrievePref, setInterval, clearInterval, setupImageButton, storePref, clearAllPrefs, fillTimeInputElem, storeAllPrefs, showGrowlAlert, setBalloonText, showBalloon, closeBalloon, parseTimeInputElem, parseIntFixed, registerWithGrowl, makeCheckboxElem, makeTimeInputElem, restoreAllPrefs */


// -----------------------------------------------------------------------------
// "Konstants"
// -----------------------------------------------------------------------------
var kMsPerMinute = 60000;	// Number of ms in a minute
var kDefaultSoundPath = "sounds/default.mp3";
var kAlarmSoundPath = "sounds/timer_alarm.mp3";


// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------
function setup()
{
	g24HourClock = is24HourClock();
	
	// Populate the sounds menu with the system & user alert sound files
    populateSoundsMenus();
	
	getElem("prefsTopBG").style.height = "223px";
	
	// Generate the buttons for the Preferences and About panes
	setupImageButton("aboutButton", "Images/about.png", "Images/about_pressed.png", aboutBtnClickHandler);
	setupImageButton("doneButton", "Images/done.png", "Images/done_pressed.png", doneButtonClickHandler);
	setupImageButton("helpButton", "Images/help.png", "Images/help_pressed.png", helpButtonClickHandler);
	setupImageButton("aboutBackButton", "Images/back.png", "Images/back_pressed.png", aboutBackBtnClickHandler);
	setupImageButton("aboutDoneButton", "Images/done.png", "Images/done_pressed.png", doneButtonClickHandler);
	setupImageButton("aboutHelpButton", "Images/help.png", "Images/help_pressed.png", helpButtonClickHandler);
	
	setupImageButton("colorButton", "Images/buttons/l_button_up.png", "Images/buttons/l_button_down.png", colorButtonClickHandler);
	setupImageButton("resetButton", "Images/buttons/r_button_up.png", "Images/buttons/r_button_down.png", resetButtonClickHandler);
	
	setupImageButton("alarmSnoozeBtn", "Images/alarm_snooze_btn.png", "Images/alarm_snooze_btn_dn.png", handleSnoozeBtnClick);
	setupImageButton("alarmOffBtn", "Images/alarm_off_btn.png", "Images/alarm_off_btn_dn.png", handleOffBtnClick);
    
	setupImageButton("showDrawerButton", "Images/expand-button-up.png", "Images/expand-button-down.png", handleShowDrawerBtnClick);
	setupImageButton("hideDrawerButton", "Images/contract-button-up.png", "Images/contract-button-down.png", handleHideDrawerBtnClick);
    
    //
    // Make some checkboxes and other stuff
    //
    makeCheckboxElem("repeatUntilDismissed", "handleRepeatUntilDismissedCheck", "Repeat until dismissed", "");
    makeCheckboxElem("showTextMessage", "handleShowTextAlertCheck", "Show text alert:", "");
    makeCheckboxElem("useGrowl", "useGrowlCheckHandler", "Use Growl", "");
    makeCheckboxElem("muteBetween", "handleMuteBetweenCheck", "Mute between", "Mute the sound between the following times");
    
    //
    // Make some time input elements
    //
    if (g24HourClock) {
        makeTimeInputElem("muteStartTime", "handleMuteBetweenStartChange", true, true, false, false, true);
        makeTimeInputElem("muteEndTime", "handleMuteBetweenEndChange", true, true, false, false, true);
        makeTimeInputElem("alarmTime", "handleClockAlarmTimeChange", true, true, false, false, true);
    } else {
        makeTimeInputElem("muteStartTime", "handleMuteBetweenStartChange", true, true, false, true, false);
        makeTimeInputElem("muteEndTime", "handleMuteBetweenEndChange", true, true, false, true, false);
        makeTimeInputElem("alarmTime", "handleClockAlarmTimeChange", true, true, false, true, false);
    }
    
	
	var i;
	for (i=0; i<4; i++) {
		gOS9Chimes[i] = null;
	}
	
	restoreAllPrefs();	// Puts the retrieved pref values into the globals
	
	
	//
	// Deal with defaults, etc.
	//
	
	// Chime type
	if (!((gChimeType == "off") || (gChimeType == "clockChimes") || 
		  (gChimeType == "os9") || (gChimeType == "alarm") || 
		  (gChimeType == "interval")))
	{
		gChimeType = "off";
	}
	setChimeType(gChimeType);	// Should this be in updateGUI()? Probably.
	
	// Volume
	gVolume = (parseInt(gVolume) && (parseInt(gVolume)>=0)) ? parseInt(gVolume) : 3;
	
	// Mute
	gIsMuted = (gIsMuted) ? true : false;
	
	// Sound path (for everything but OS 9 sounds)
	gPathToSound = (gPathToSound && (gPathToSound.length>0)) ? gPathToSound : null;
	if (gPathToSound) {
		setSoundByPath(gPathToSound);
	} else {
		setSoundById(0);	// Use the default sound if a stored pref wasn't found
	}
	
	// Background color
	gLastBGColor = (gLastBGColor && (gLastBGColor>0) && (gLastBGColor<=5)) ? gLastBGColor : 1;
	setBGColor(gLastBGColor);
    
    //
    // Like it says, register with Growl
    //
    gGrowlExists = registerWithGrowl();
	
	//
	// Finish startup
	//
	// Check to make sure that we are running in Dashboard
	if (window.widget) {
		setVol(gVolume);
		setMute(gIsMuted);
		setInterval("periodicStuff()", 15000);	// Execute every 15 seconds
		startChimes();	// Actually get chimes going
		updateGUI();
        if (gDrawerShown) {
            moveDrawer(true, 160);
        }
	}
	
	//toggleDebug();
    
    setBalloonText("ProdMe Alert", gTextMessage, false);
}



// -----------------------------------------------------------------------------
// Event handlers
// -----------------------------------------------------------------------------

function handleRepeatUntilDismissedCheck(elem)
{
    gAlarmRepeatUntilDismissed = elem.checked;
    storeAllPrefs();
}

function handleAlarmTimeChange(elem)
{
    gAlarmTime = (parseTimeInputElem("alarmTime")).getTime();
    storeAllPrefs();
}


function handleMuteBetweenCheck(elem)
{
    gMuteBetweenEnabled = elem.checked;
    storeAllPrefs();
}


function handleMuteBetweenStartChange(elem)
{
    gMuteBetweenStart = (parseTimeInputElem("muteStartTime")).getTime();
    storeAllPrefs();
}


function handleMuteBetweenEndChange(elem)
{
    gMuteBetweenEnd = (parseTimeInputElem("muteEndTime")).getTime();
    storeAllPrefs();
}


function handleCustomTimeOnBlur()
{
    var minutes = parseIntFixed(getElem("customTimeMin").value);
    var hours = parseIntFixed(getElem("customTimeHours").value);
    gOtherCustomInterval = minutes + (hours*60);
    if (gOtherCustomInterval <= 0) {
        gOtherCustomInterval = 1;
    }
    storeAllPrefs();
}


function handleClockAlarmTimeChange(elem)
{
    gAlarmTime = (parseTimeInputElem("alarmTime")).getTime();
    storeAllPrefs();
}

function handleOtherChimeTypeChange()
{
    gOtherUseCustomInterval = getElem('intervalCustom').checked;
    storeAllPrefs();
}


function chimeChoiceMenuHandler(elem)
{
	var menuChoice = elem.options[elem.selectedIndex].value;
	setChimeType(menuChoice);
	storeAllPrefs();
}


function setChimeType(chimeType)
{
	//
	// Hide everything
	//
	getElem("os9Pane").style.visibility = "hidden";
	getElem("intervalPane").style.visibility = "hidden";
	getElem("soundsPane").style.visibility = "hidden";
	getElem("chimesPane").style.visibility = "hidden";
	getElem("alarmPane").style.visibility = "hidden";
	getElem("volumePane").style.visibility = "hidden";
	getElem("offDescLabel").style.visibility = "hidden";
	
	//
	// Find the target size of the widget
	//
	var targetHeight = null;
	
	switch (chimeType) {
		case "clockChimes":
			targetHeight = 164;
			break;
			
		case "os9":
			targetHeight = 212;
			break;
			
		case "alarm":
			targetHeight = 187;
			break;
			
		case "interval":
			targetHeight = 197;
			break;
			
		case "about":
			targetHeight = 183;
            // Hide the options drawer
            if (gDrawerShown) {
                moveDrawer(true, 0);
            } else {
                moveDrawer(false, 0);
            }
			break;
		
		default:	// "off"
			targetHeight = 120;
			chimeType = "off";	// normalize things, if necessary
            // Hide the options drawer
            if (gDrawerShown) {
                moveDrawer(true, 0);
            } else {
                moveDrawer(false, 0);
            }
			break;
	}
	
	//
	// Set the global state
	//
	if (chimeType != "about") {	// kludge kludge kludge kludge...
		gChimeType = chimeType;
	}
    
    //
    // Show the drawer when appropriate
    //
	if ((chimeType != "about") && (chimeType != "off")) {
        if (gDrawerShown) {
            moveDrawer(true, 160);
        } else {
            moveDrawer(false, 37);
        }
	}
	
	//
	// Resize the widget
	//
	//smoothResize(209, targetHeight);
	smoothResize(gCurrentBackWidth, targetHeight);
}



function setChimeTypeCleanup()
{
	//
	// Deal with About pane kludge
	//
	if (gShowingAbout) {
		getElem("back").style.display = 'none';
		getElem("about").style.display = 'block';
		
		var theDiv = getElem("babble");
		theDiv.style.display = 'block';
		calculateAndShowThumb(theDiv);
		
		gShowingAbout = false;
	}
	
	//
	// Move & show stuff as needed
	//
	if (gChimeType == "off") {
		getElem("offDescLabel").style.visibility = "visible";
	}
	else if (gChimeType == "clockChimes") {
		getElem("soundsPane").style.top = "123px";
		getElem("soundsPane").style.visibility = "visible";
		getElem("volumePane").style.top = "141px";
		getElem("volumePane").style.visibility = "visible";
		getElem("chimesPane").style.visibility = "visible";
	}
	else if (gChimeType == "os9") {
		getElem("volumePane").style.top = "188px";
		getElem("volumePane").style.visibility = "visible";
		getElem("os9Pane").style.visibility = "visible";
	}
	else if (gChimeType == "interval") {
		getElem("soundsPane").style.top = "156px";
		getElem("soundsPane").style.visibility = "visible";
		getElem("volumePane").style.top = "174px";
		getElem("volumePane").style.visibility = "visible";
		getElem("intervalPane").style.visibility = "visible";
	}
	else if (gChimeType == "alarm") {
		//getElem("soundsPane").style.top = "176px";
		getElem("soundsPane").style.top = "146px";
		getElem("soundsPane").style.visibility = "visible";
		//getElem("volumePane").style.top = "200px";
		getElem("volumePane").style.top = "164px";
		getElem("volumePane").style.visibility = "visible";
		getElem("alarmPane").style.visibility = "visible";
	}
}



var gTargetSize;
var gStepSize;
var gSmoothResizeInterval;
function smoothResize(width, height)
{
	var topElem = getElem("prefsTopBG");
	var currentSize = parseInt(topElem.style.height);
	var prefsPane = getElem("back");
    var drawerPane = getElem("prefsDrawer");
	
	// If the preferences aren't visible anyway, there's no reason to do this slowly.
	if (prefsPane.style.display != 'block') {
		if (height != currentSize) {
			gTargetSize = height;
			var bottomElem = getElem("prefsButtonsPane");
			topElem.style.height = gTargetSize + "px";
            drawerPane.style.height = (gTargetSize+26) + "px";
			bottomElem.style.top = topElem.style.height;
		}
		setChimeTypeCleanup();
	}
	else {
		if (height != currentSize) {
			var stepCount = 7;
			gTargetSize = height;
			gStepSize = (currentSize - height) / stepCount;
			gSmoothResizeInterval = setInterval("doSmoothResize();", 13);
		}
	}
}


var gStepCount = 0;
function doSmoothResize()
{
	var topElem = getElem("prefsTopBG");
	var bottomElem = getElem("prefsButtonsPane");
    var drawerPane = getElem("prefsDrawer");
	
	var currentHeight = parseFloat(topElem.style.height);
	var newHeight = (currentHeight - gStepSize);
	topElem.style.height = newHeight + "px";
    drawerPane.style.height = (newHeight+26) + "px";
	bottomElem.style.top = topElem.style.height;
	gStepCount++;
	
	if ((gStepCount > 50) ||
		((gStepSize < 0) && (newHeight >= gTargetSize)) ||
		((gStepSize > 0) && (newHeight <= gTargetSize)))
	{
		clearInterval(gSmoothResizeInterval);
		topElem.style.height = gTargetSize + "px";
        drawerPane.style.height = (gTargetSize+26) + "px";
		bottomElem.style.top = topElem.style.height;
		gStepCount = 0;
		// Should do this as a callback, but oh well...
		setTimeout("setChimeTypeCleanup();", 13);
	}
}




function getCustomTime()
{
	var minVal = getElem("customTimeMin").value;
	var min = 0;
	// Now this is just weird.  parseInt() seems to have a blind spot for 8 and
	// 9 when a string with either digit in it is preceeded by a 0.  This bug
	// is easy to reproduce.  I'll submit a report to Apple when I have time.
	/*if (minVal == "08") {
		min = 8;
	} else if (minVal == "09") {
		min = 9;
	} else {
		min = parseInt(getElem("customTimeMin").value);
	}*/
    min = parseIntFixed(minVal);
	if ((!min) || (min<0)) { min = 0; }
    var hours = parseIntFixed(getElem("customTimeHours").value);
	if ((!hours) || (hours<0)) { hours = 0; }
	return min + (hours * 60);
}


/**
 * Handler for the time interval popup menu
 */
function intervalMenuHandler(elem)
{
	getElem("intervalPreset").checked = true;
	gOtherPresetInterval = parseInt(elem.options[elem.selectedIndex].value);
	storeAllPrefs();
}


/**
 * Handler for the clock chimes popup menu.
 */
function chimesMenuHandler(elem)
{
	gClockChimeInterval = parseInt(elem.options[elem.selectedIndex].value);
	storeAllPrefs();
} 


/**
 * Handler for the volume popup menu.
 */
function volumeMenuHandler(elem)
{
	var level = parseInt(elem.options[elem.selectedIndex].value);
	setVol(level);
	setMute(false);
	playSound(getVolumeSoundPath());
	updateGUI();
}


/**
 * Handler for the volume control on the front.
 */
function volumeClickHandler(level)
{
	setMute(false);
	setVol(level);
	updateGUI();
	playSound(getVolumeSoundPath());
}


/**
 * Mute checkbox click event handler.
 */
function muteClickHandler(elem)
{
	var muted_p = elem.checked;
	setMute(muted_p);
	if (!muted_p) {
		playSound(getVolumeSoundPath());
	}
}


/**
 * Handles selection of a sound from one of the OS9 chimes menus.
 */
function OS9SoundsMenuHandler(selection, menuId)
{
	var selectedItem = selection.value;
	
	if (selectedItem == 999) {
		gOS9Chimes[menuId] = null;
	} else {
		gOS9Chimes[menuId] = gSoundNamesPaths[selectedItem][1];
		playSound(gOS9Chimes[menuId]);
	}
	
	storeAllPrefs();
}


/**
 * Handles the selection of a sound from the Sound menu.
 */
function soundsMenuHandler(selection)
{
	// Set the sound
	setSoundByPath(gSoundNamesPaths[selection.value][1]);
	
	// Unmute if necessary
	setMute(false);
	
	// Play the selected sound
	playSound(gPathToSound);
}


/**
 * Event handler for both Done buttons.
 */
function doneButtonClickHandler()
{
	updateStateFromGUI();
	storeAllPrefs();
	startChimes();
	updateGUI();
	hidePrefs();
}


/**
 * Event handler for the ? (help) button
 */
function helpButtonClickHandler()
{
	widget.openURL('http://cruftforce7.com/faq.html');
}


/**
 * Event handler for the About button on the preferences pane
 */
var gShowingAbout = false;
function aboutBtnClickHandler()
{
	gShowingAbout = true;
	setChimeType("about");
}


/**
 * Event handler for the Back button on the About pane
 */
function aboutBackBtnClickHandler()
{
	getElem("about").style.display = 'none';
	getElem("back").style.display = 'block';
	setChimeType(gChimeType);
}


var gSmoothSetBGColorInterval = null;
var gSmoothSetBGColorState = 0.0;
/**
 * Fades between colors on the front face of the widget.
 */
function smoothSetBGColor(color)
{
	if (gSmoothSetBGColorInterval !== null) {
		return;
	}
	gSmoothSetBGColorState = 0.0;
	getElem("backlightFade").style.opacity = 0.0;
	
	switch(color) {
		case 1:
			getElem("backlightFade").src = "Images/backlights/green.png";
			break;
			
		case 2:
			getElem("backlightFade").src = "Images/backlights/blue.png";
			break;
		
		case 3:
			getElem("backlightFade").src = "Images/backlights/purple.png";
			break;
		
		case 4:
			getElem("backlightFade").src = "Images/backlights/orange.png";
			break;
		
		case 5:
			getElem("backlightFade").src = "Images/backlights/yellow.png";
			break;
	}
	
	gSmoothSetBGColorInterval = setInterval("doSmoothSetBGColor(" + color + ");", 13);
}

/**
 * Performs the steps of the color fading for the front face.
 */
function doSmoothSetBGColor(color)
{
	var opacityInc = 0.05;
	if (gColorBtnShiftDown) {
		opacityInc = 0.005;
	}
	gSmoothSetBGColorState += opacityInc;
	if (gSmoothSetBGColorState > 1.0) {
		gSmoothSetBGColorState = 1.0;
	}
	
	getElem("backlightFade").style.opacity = gSmoothSetBGColorState;
	getElem("backlight").style.opacity = (1.0 - gSmoothSetBGColorState);
	if (gSmoothSetBGColorState == 1.0) {
		clearInterval(gSmoothSetBGColorInterval);
		setBGColor(color);
		getElem("backlightFade").style.opacity = 0.0;
		getElem("backlight").style.opacity = 1.0;
		gSmoothSetBGColorInterval = null;
	}
}



var gColorBtnShiftDown = false;
/**
 * Cycle through the colors of the front face of the widget; called from the
 * onClick event of the color button.
 */
function colorButtonClickHandler(event)
{
	gLastBGColor++;
	if (gLastBGColor > 5) {
		gLastBGColor = 1;
	}
	
	if (event.shiftKey) {
		gColorBtnShiftDown = true;
	} else {
		gColorBtnShiftDown = false;
	}
	smoothSetBGColor(gLastBGColor);
}


function handleSnoozeBtnClick()
{
    startSnooze();
}


function handleOffBtnClick()
{
    stopAlarm();
}

function printPropertyNames(obj)
{
    var names = "";
    for (var name in obj) { names += name + ", "; }
    alert(names);
}

// Event object properties: stopPropagation, preventDefault, initEvent, metaKey, keyIdentifier, altKey, keyLocation, ctrlKey, shiftKey, altGraphKey, charCode, view, keyCode, layerX, layerY, detail, pageX, pageY, which, timeStamp, cancelBubble, bubbles, returnValue, type, srcElement, target, currentTarget, eventPhase, cancelable, dataTransfer, clipboardData

function handleNumberKeyPress(elem)
{
    var character = String.fromCharCode(event.keyCode);
    
    if ((isNaN(parseInt(character))) &&
        !(((event.keyCode <= 40) && (event.keyCode >= 37)) ||   // Arrow keys
          (character == "\t") ||                                // Tab
          (event.keyCode === 0) ||                                // Tab
          (character == "\r") ||                                // Return
          (character == "\b") ||                                // Backspace
          event.metaKey
         )
        )
    {
        event.returnValue = false;
    }
}


function handleNumberOnBlur(elem, val, min, max, fillZeros)
{
    var userVal = parseIntFixed(elem.value);
    var finalVal = userVal;
    
    if (isNaN(userVal)) {
        finalVal = val;
    }
    else if (userVal < min) {
        finalVal = min;
    } else if (userVal > max) {
        finalVal = max;
    }
    
    if (fillZeros && ((finalVal >= 0) && (finalVal < 10))) {
        finalVal = "0" + finalVal;
    }
    
    elem.value = finalVal;
}



function handleShowDrawerBtnClick()
{
    moveDrawer(true, 160);
    setTimeout("storeAllPrefs();", 1500);   // *** KLUDGE!!!!
}

function handleHideDrawerBtnClick()
{
    moveDrawer(false, 37);
    setTimeout("storeAllPrefs();", 1500);   // *** KLUDGE!!!!
}


var gMoveDrawerInc = null;
var gMaxBackWidth = 360;
var gTargetDrawerWidth = null;
var gDrawerResizeIntervalId = null;
var gDrawerExpand = false;  // *** Is this necessary?
function moveDrawer(expand, width)
{
    gDrawerExpand = expand;
    gTargetDrawerWidth = width;
    var drawer = getElem("prefsDrawer");
    var currentDrawerWidth = parseIntFixed(drawer.style.width);
    var back = getElem("back");
    gDrawerShown = expand;
    if (getElem("front").style.display == 'none') {
        if (currentDrawerWidth != gTargetDrawerWidth) {
            gMoveDrawerInc = parseInt((width - currentDrawerWidth) / 10);   // 10 steps
            if (expand) {
                window.resizeTo(gMaxBackWidth, 280);
                back.style.width = gMaxBackWidth + "px";
                gCurrentBackWidth = gMaxBackWidth;
            }
            
            gDrawerResizeIntervalId = setInterval("doMoveDrawer();", 13);
        }
    } else {
        drawer.style.width = width + "px";
        if (expand) {
            back.style.width = gMaxBackWidth + "px";
            gCurrentBackWidth = gMaxBackWidth;
            getElem("showDrawerButton").style.display = "none";
            getElem("hideDrawerButton").style.display = "block";
        } else {
            back.style.width = gDefaultBackWidth + "px";
            gCurrentBackWidth = gDefaultBackWidth;
            getElem("hideDrawerButton").style.display = "none";
            getElem("showDrawerButton").style.display = "block";
        }
    }
}


function doMoveDrawer()
{
    var drawer = getElem("prefsDrawer");
    var currentDrawerWidth = parseIntFixed(drawer.style.width);
    var newWidth = currentDrawerWidth + gMoveDrawerInc;
    if (((newWidth >= gTargetDrawerWidth) && (gMoveDrawerInc >= 0)) ||
        ((newWidth <= gTargetDrawerWidth) && (gMoveDrawerInc < 0)))
    {
        clearInterval(gDrawerResizeIntervalId);
        drawer.style.width = gTargetDrawerWidth + "px";
        setTimeout("moveDrawerCleanup()", 50);
    } else {
        drawer.style.width = newWidth + "px";
    }
}

function moveDrawerCleanup()
{
    if (gDrawerExpand) {
        getElem("showDrawerButton").style.display = "none";
        getElem("hideDrawerButton").style.display = "block";
    } else {
        getElem("hideDrawerButton").style.display = "none";
        getElem("showDrawerButton").style.display = "block";
        getElem("back").style.width = gDefaultBackWidth + "px";
        gCurrentBackWidth = gDefaultBackWidth;
        window.resizeTo(gDefaultBackWidth, 280);
        //gDrawerShown = false;
    }
}



/**
 * Set the color of the front face of the widget
 */
function setBGColor(color)
{
	switch(color) {
		case 1:
			getElem("backlight").src = "Images/backlights/green.png";
			break;
			
		case 2:
			getElem("backlight").src = "Images/backlights/blue.png";
			break;
		
		case 3:
			getElem("backlight").src = "Images/backlights/purple.png";
			break;
		
		case 4:
			getElem("backlight").src = "Images/backlights/orange.png";
			break;
		
		case 5:
			getElem("backlight").src = "Images/backlights/yellow.png";
			break;
	}
	
	storePref(color, "bgColor");
}


/**
 * Restart the countdown timer (when using arbitrary intervals; has no effect
 * otherwise).
 */
function resetButtonClickHandler()
{
    stopChimes();
	startChimes();
	updateStateFromGUI();
	updateGUI();
}


function toggleChimesStrikeHour()
{
    gClockChimeStrikeHour = !gClockChimeStrikeHour;
	getElem("hourRepeatChimesCheck").checked = gClockChimeStrikeHour;
	storeAllPrefs();
}


function toggleOS9StrikeHour()
{
    gOS9StrikeHour = !gOS9StrikeHour;
	getElem("hourRepeatOS9Check").checked = gOS9StrikeHour;
	storeAllPrefs();
}

function handleChimesStrikeHourCheck(elem)
{
	getElem("hourRepeatChimesCheck").checked = elem.checked;
	gClockChimeStrikeHour = elem.checked;
	storeAllPrefs();
}

function handleOS9StrikeHourCheck(elem)
{
	getElem("hourRepeatOS9Check").checked = elem.checked;
	gOS9StrikeHour = elem.checked;
	storeAllPrefs();
}


function useGrowlCheckHandler(elem)
{
    if (gGrowlExists) {
        gUseGrowl = elem.checked;
        storeAllPrefs();
    }
}


function handleShowTextAlertCheck(elem)
{
    gShowTextMessage = elem.checked;
    storeAllPrefs();
}


// -----------------------------------------------------------------------------
// Volume control & mute
// -----------------------------------------------------------------------------

/**
 * Sets the volume to a level between 0 and 6 (0 is off, but gIsMuted is not set)
 */
function setVol(level)
{
	// Do a bounds check
	if ((level > 6) || (level < 0)) {
		return;
	}
	
	// Actually set the level
	gVolume = level;
	
	// Set the preference for this volume level
	storePref(gVolume, "volume");
}


/**
 * Toggles the mute on and off.  Plays the chosen sound when mute is turned off.
 */
function toggleMute()
{
	if (gIsMuted) {
		setMute(false);
		playSound(getVolumeSoundPath());
	} else {
		setMute(true);
	}
}


/**
 * Sets mute on or off, based on mutep.
 * @param mutep A boolean. True turns mute on, false turns it off.
 */
function setMute(mutep)
{
	gIsMuted = mutep;
	if (mutep) {
		stopPlayer();
		storeAllPrefs();
		storePref(true, "isMuted");
	} else {
		storePref(false, "isMuted");
	}
	
	updateGUI();
}



// -----------------------------------------------------------------------------
// Time display
// -----------------------------------------------------------------------------

/**
 * Given a time interval in minutes, figures out what needs to be displayed to
 * the user, and passes that info along to setAlertDisplay().  That function
 * should probably be rolled together with this one, but I'm lazy.
 * @param chimeTime The time of the next chime, as a Date object.
 */
function setAlertTime(chimeTime)
{
    //
    // If the argument is null, clear the display
    //
	if (!chimeTime) {
		setAlertDisplay(-1, -1, "");
		return;
	}
	
    //
    // Figure out what to display based on the user's preferences
    //
    var hoursDisplay = chimeTime.getHours();
    var amPm = "off";
    if (!g24HourClock) {
        //
        // If a 12 hour clock is in use, adjust the time display accordingly
        //
		if (hoursDisplay > 12) {
			hoursDisplay -= 12;
		}
		if (hoursDisplay === 0) {
			hoursDisplay = 12;
		}
        amPm = (chimeTime.getHours()>11) ? "pm" : "am";
	}
	
	setAlertDisplay(hoursDisplay, chimeTime.getMinutes(), amPm);
}


/**
 * Displays a time on the front pane of the widget.
 * @param hours the hour, between 1 and 12
 * @param minutes the minutes, between 0 and 59
 * @param amPm a string, "am" or "pm"
 */
function setAlertDisplay(hours, minutes, amPm)
{
	if ((hours > 99) || (minutes > 59) || (hours < 0) || (minutes < 0)) {
		getElem("numPos4").src = "Images/blank.png";
		getElem("numPos3").src = "Images/blank.png";
		getElem("numPos2").src = "Images/blank.png";
		getElem("numPos1").src = "Images/blank.png";
		getElem("am").src = "Images/blank.png";
		getElem("pm").src = "Images/blank.png";
		getElem("colon").src = "Images/blank.png";
		getElem("nextChime").src = "Images/blank.png";
		getElem("chimeNotSet").src = "Images/lcd/chime_not_set.png";
		return;
	}
	
	var min1   = minutes % 10;				// right hand digit of minutes
	var min2   = Math.floor(minutes/10);	// left hand digit of minutes
	var hours1 = hours % 10;				// right hand digit of hours
	var hours2 = Math.floor(hours/10);		// left hand digit of hours
	
	getElem("chimeNotSet").src = "Images/blank.png";
	getElem("colon").src = "Images/lcd/colon.png";
	getElem("nextChime").src = "Images/lcd/next_chime.png";
	
	getElem("numPos4").src = "Images/lcd/numbers/num" + min1 + ".png";
	getElem("numPos3").src = "Images/lcd/numbers/num" + min2 + ".png";
	getElem("numPos2").src = "Images/lcd/numbers/num" + hours1 + ".png";
	if (hours2 > 0) {
		getElem("numPos1").src = "Images/lcd/numbers/num" + hours2 + ".png";
	} else {
		getElem("numPos1").src = "Images/blank.png";
	}
	
	if (amPm == "pm") {
		getElem("am").src = "Images/blank.png";
		getElem("pm").src = "Images/lcd/pm.png";
	} else if (amPm == "am") {
		getElem("am").src = "Images/lcd/am.png";
		getElem("pm").src = "Images/blank.png";
	} else {
		getElem("am").src = "Images/blank.png";
		getElem("pm").src = "Images/blank.png";
	}
}



// -----------------------------------------------------------------------------
// Sound-related stuff
// -----------------------------------------------------------------------------

var gSoundsList = null;
/**
 * Determines if the sound menus need to be updated, and if so, does it.
 */
function populateSoundsMenus()
{
    var i = 0;
    
    if (!gSoundNamesPaths) {
		gSoundNamesPaths = new Array;
	}
    // Get the current username
    var currentUser = getUser();
    
    // Get lists of sound file names and paths from both the user and system
    // alert sound directories
    var grepPattern      = " | grep -E '\.aiff|\.m4a|\.aif|\.wav|\.mp3'";
    var systemSoundsPath = "/System/Library/Sounds/";
    var librarySoundsPath = "/Library/Sounds/";
    var userSoundsPath	 = "/Users/" + currentUser + "/Library/Sounds/";
    var grepawkPath	     = grepPattern + " | awk '{print";
    
    var getNamesCommand  = "ls " + userSoundsPath + grepPattern + "; ls " + systemSoundsPath + grepPattern + "; ls " + librarySoundsPath + grepPattern;
    
    var getPathsCommand  = "ls " + userSoundsPath + grepawkPath + '"' + userSoundsPath + '"' + "$0}'" + "; ls " + systemSoundsPath + grepawkPath + '"' + systemSoundsPath + '"' + "$0}'" + "; ls " + librarySoundsPath + grepawkPath + '"' + librarySoundsPath + '"' + "$0}'";
    if (window.widget) {
        // Run shell to get names and paths in two long strings
        var names = widget.system(getNamesCommand, null).outputString;
        
        //
        // If we need to update, do so.
        //
        if (gSoundsList != names) {
            gSoundNamesPaths = new Array;
            gSoundsList = names;
            var paths = widget.system(getPathsCommand, null).outputString;
            
            // Split the sound names and paths into array entries
            var soundNameArray = names.split('\n');
            var soundPathArray = paths.split('\n');
            
            // Strip the extension from each filename, then add the entry to the array
            // soundNameArray.length-1 is used because the last element of the arrays is empty
            for (i=0; i<(soundNameArray.length-1); i++) {
                var soundName = soundNameArray[i];
                var soundPath = "file://" + escape(soundPathArray[i]);
                soundName = soundName.substring(0, soundName.lastIndexOf('.'));
                gSoundNamesPaths[i] = new Array(soundName, soundPath);
            }
			
			// Put the entries in alphabetical order
			gSoundNamesPaths.sort(compareSoundNames);
			
			// Insert the default sounds
            var alarmSndPath = getWidgetRoot() + kAlarmSoundPath;
			gSoundNamesPaths.unshift(new Array("Annoying Alarm", alarmSndPath));
			var defSndPath = getWidgetRoot() + kDefaultSoundPath;
			gSoundNamesPaths.unshift(new Array("Default", defSndPath));
            
            //
            // Update the individual sound menus
            //
            populateSoundsMenu(getElem("soundsMenu"));
            populateSoundsMenu(getElem("OS9HourSoundsMenu"), true);
            populateSoundsMenu(getElem("OS915PastSoundsMenu"), true);
            populateSoundsMenu(getElem("OS9HalfSoundsMenu"), true);
            populateSoundsMenu(getElem("OS945PastSoundsMenu"), true);
        }
    }
}


/**
 * Generate the content for the sound items menu (adapted from YOLK widget)
 */
function populateSoundsMenu(soundMenu, includeNoChimeOption)
{
	// Populate the sound menu, clearing the existing items first
    clearChildren(soundMenu);
    
    var element = null;
    if (includeNoChimeOption) {
        element = document.createElement("option");
        element.innerText = "No chime";
		element.setAttribute("value", 999);
		soundMenu.appendChild(element);
    }
    
    var i = 0;
	for (i=0; i<gSoundNamesPaths.length; i++) {
		element = document.createElement("option");
		element.innerText = gSoundNamesPaths[i][0];
		element.setAttribute("value", i);
		soundMenu.appendChild(element);
	}
}



/**
 * Removes all the child nodes of a DOM node
 */
function clearChildren(n)
{
    var c = n.childNodes;
    while (n.hasChildNodes()) {
        n.removeChild(c[0]);
    }
}



/**
 * Sort comparison function for sorting the list of sound names/paths
 */
function compareSoundNames(item1, item2)
{
	if ((item1[0]).toLowerCase() > (item2[0]).toLowerCase()) {
		return 1;
	} else {
		return -1;
	}
}


/**
 * Sets the current sound via its path (as opposed to the menu item ID, for
 * example).  Useful for saved preferences.
 */
function setSoundByPath(soundPath)
{
	gPathToSound = soundPath;
	
	// Store the preference for the sound
	storePref(gPathToSound, "sound");
}


function setSoundById(id)
{
	// Set the sound
	setSoundByPath(gSoundNamesPaths[id][1]);
}

/**
 * Returns the path to the sound that should be played when the volume is
 * adjusted or mute is turned off.
 */
function getVolumeSoundPath()
{
	var i;
	if (gChimeType=="os9") {
		for (i=0; i<4; i++) {
			if (gOS9Chimes[i]) {
				return gOS9Chimes[i];
			}
		}
	}
	
	return gPathToSound;
}


var gPlaySoundFailed = false;
var gPlayStopTimeoutID = null;
var gPlayRepeatTimeoutID = null;
var gPlayCount = 0;		// For debugging purposes.  # of times the function has been called, aside from when sound is muted.
/**
 * Plays the chime sound (assumes that the QuickTime plugin already has the
 * sound path).  Won't play if the user has muted the sound.
 * @param soundPath full path to the sound
 * @param repeatCount (optional) number of times to play the sound
 * @param repeatDelay (optional) amount of delay (in ms) between repeats 
 */
function playSound(soundPath, repeatCount, repeatDelay)
{
	if (gIsMuted) {
		//
		// The sound is muted, so don't do anything
		//
		return;
	}
	
	repeatCount = (parseIntFixed(repeatCount)) ? parseIntFixed(repeatCount) : 1;
	repeatDelay = (parseIntFixed(repeatDelay)) ? parseIntFixed(repeatDelay) : 0;
	
	if (gPlayStopTimeoutID) {
		clearTimeout(gPlayStopTimeoutID);
		gPlayStopTimeoutID = null;
	}
	var vol = parseInt((gVolume/6.0) * 100);
	
	// 
	// Create the QuickTime plugin to play the sound
	//
	var qtPlayer = getElem("playerQT");
	if (!qtPlayer) {
		getElem("qtPluginPlaceholder").innerHTML = "<embed id='playerQT' src='"+
			soundPath + "' hidden=true autostart=false volume=" + vol + " loop=false>";
		qtPlayer = getElem("playerQT");
        qtPlayer.SetVolume(vol);
		qtPlayer.Play();
	} else {
		qtPlayer.SetURL(soundPath);
		qtPlayer.SetVolume(vol);
		qtPlayer.Play();
	}
	//
	// Check for errors
	//
	var pluginStatus = qtPlayer.GetPluginStatus();
	if (pluginStatus.substring(0, 5) != "Error") {
		//
		// No errors reported.
		//
		gPlaySoundFailed = false;
		var durationInSeconds = (qtPlayer.GetDuration() / qtPlayer.GetTimeScale());
		if (repeatCount > 1) {
            //
			// Play the sound again
            //
            var playSndStr = "";
            if (gAlarmOn && !gAlarmSnoozeOn) {
                //
                // Repeat indefinitely, until alarm is turned off
                //
                playSndStr = "playSound(\"" + soundPath + "\", " + 
                    2 + ", " + 50 + ")";
                gPlayRepeatTimeoutID = setTimeout(playSndStr, (durationInSeconds * 1000) + 50);
            } else {
                playSndStr = "playSound(\"" + soundPath + "\", " + 
                    (repeatCount-1) + ", " + repeatDelay + ")";
                gPlayRepeatTimeoutID = setTimeout(playSndStr, (durationInSeconds * 1000) + repeatDelay);
            }
		}
		else {
			// This may not be necessary anymore...
			gPlayStopTimeoutID = setTimeout("stopPlayer()",
				((durationInSeconds + 1) * 1000));
		}
	} 
	else if (!gPlaySoundFailed) {
		//
		// An error occurred
		//
		stopPlayer();
		alert("QuickTime plugin reported an error: " + pluginStatus);
		setSoundById(0);			// Pick the default sound
		updateGUI();
		gPlaySoundFailed = true;
		playSound(gPathToSound);	// Try again
	}
	else {
		//
		// More errors. Playing the default sound didn't work, either.
		//
		stopPlayer();
		alert("QuickTime plugin reported an error (could not recover)");
		gPlaySoundFailed = false;	// We can try again at next call
	}
	
	return;
}


function stopPlayer()
{
	var qtPlayer = getElem("playerQT");
	if (qtPlayer) {
		qtPlayer.Stop();
	}
    clearTimeout(gPlayStopTimeoutID);   // This might not be needed if this function was called from the timeout itself, but it shouldn't hurt
	gPlayStopTimeoutID = null;
    clearTimeout(gPlayRepeatTimeoutID);
    gPlayRepeatTimeoutID = null;
}






// -----------------------------------------------------------------------------
// Preferences
// -----------------------------------------------------------------------------

/**
 * Delete this instance's preference settings when it is removed from Dashboard.
 */
function onremove()
{
	clearAllPrefs();
}


// -----------------------------------------------------------------------------
// Widget state
// -----------------------------------------------------------------------------

function disableResetButton()
{
	getElem("resetLbl").style.display = "none";
	disableButton("resetButton");
	getElem("resetButton").title = "Reset the timer (disabled because clock-synchronized chimes are in use)";
}

function enableResetButton()
{
	getElem("resetLbl").style.display = "block";
	enableButton("resetButton");
	getElem("resetButton").title = "Reset the timer";
}

/**
 * Updates the user interface elements based on the state of the global
 * variables.  This would be harmfully inefficient if it were running on a
 * 20 year old computer.
 */
function updateGUI()
{
	var i;
	
	//
	// Chime type menu
	//
	getElem("chimesChoiceMenu").value = gChimeType;
	
	//
	// If the time interval is 0, choose the "none" preset menu option
	// regardless of what the user chose.
	//
	if (gChimeType == "off") {
		disableResetButton();
		setAlertTime(null);
	}
	
	//
	// Set up the panes correctly
	//
	if (gChimeType == "clockChimes") {
		disableResetButton();
		getElem("chimesMenu").value = gClockChimeInterval;
	}
    
	if (gChimeType == "interval") {
		enableResetButton();
        getElem("intervalMenu").value = gOtherPresetInterval;
		if (!gOtherUseCustomInterval) {
			getElem("intervalPreset").checked = true;
		} else {
			getElem("intervalCustom").checked = true;
			// We finish the rest down below...
		}
        // #### Now we should finish everything here instead of below
	}
    
	if (gChimeType == "os9") {
		disableResetButton();
	}
    
	if (gChimeType == "alarm") {
        disableResetButton();
        if (gAlarmRepeatUntilDismissed) {
            getElem("alarmButtons").style.visibility = "visible";
        } else {
            getElem("alarmButtons").style.visibility = "hidden";
        }
	} else {
        getElem("alarmButtons").style.visibility = "hidden";
    }
	
	
	//
	// Select the appropriate sound in the popup menu
	//
	var soundMenuElem = getElem("soundsMenu");
	for (i=0; i<gSoundNamesPaths.length; i++) {
		if (gSoundNamesPaths[i][1] == gPathToSound) {
			soundMenuElem.value = i;
			break;
		}
	}
	// Same thing for OS 9 chimes
	var j;
	for (j=0; j<4; j++) {
		switch (j) {
			case 0:
				soundMenuElem = getElem("OS9HourSoundsMenu");
				break;
			case 1:
				soundMenuElem = getElem("OS915PastSoundsMenu");
				break;
			case 2:
				soundMenuElem = getElem("OS9HalfSoundsMenu");
				break;
			case 3:
				soundMenuElem = getElem("OS945PastSoundsMenu");
				break;
		}
		if (gOS9Chimes[j]) {
			for (i=0; i<gSoundNamesPaths.length; i++) {
				if (gSoundNamesPaths[i][1] == gOS9Chimes[j]) {
					soundMenuElem.value = i;
					break;
				}
			}
		}
	}
	
	//
	// Set the custom time interval text fields
	//
	if (true) {
		var mins = gOtherCustomInterval % 60;
		var hours = Math.floor(gOtherCustomInterval/60);
		var minsStr = "00";
		var hoursStr = "0";
		if ((mins>0) && (mins<10) && (hours>0)) {
			minsStr = "0" + mins;
		} else if (mins>0) {
			minsStr = mins;
		}
		//if (hours>0) {
		if (true) {
			hoursStr = hours;
		}
		if (gOtherCustomInterval === 0) {
			hoursStr = "0";
			minsStr = "01";
		}
		getElem("customTimeMin").value = minsStr;
		getElem("customTimeHours").value = hoursStr;
	}
    
    //
    // Alarm clock settings
    //
    var alarmDate = new Date(gAlarmTime);
    fillTimeInputElem("alarmTime", alarmDate);
    getElem("alarmSnoozeDelay").value = gAlarmSnoozeLength;
    getElem("repeatUntilDismissedCheck").checked = gAlarmRepeatUntilDismissed;
    
    //
    // Text notification message
    //
    getElem("showTextMessageCheck").checked = gShowTextMessage;
    getElem("alarmMessage").value = gTextMessage;
    if (gGrowlExists) {
        getElem("useGrowl").style.display = "block";
        getElem("useGrowlCheck").checked = gUseGrowl;
    } else {
        getElem("useGrowl").style.display = "none";
        getElem("useGrowlCheck").checked = false;
    }
    
    getElem("muteBetweenCheck").checked = gMuteBetweenEnabled;
    fillTimeInputElem("muteStartTime", new Date(gMuteBetweenStart));
    fillTimeInputElem("muteEndTime", new Date(gMuteBetweenEnd));
    
	//
	// Repeat chime on the hour (where applicable)
	//
	getElem("hourRepeatOS9Check").checked = gOS9StrikeHour;
	getElem("hourRepeatChimesCheck").checked = gClockChimeStrikeHour;
	
	//
	// Sound volume display settings
	//
	
	// Update the front display
	var btnStr;
	for (i=1; i<=gVolume; i++) {
		btnStr = "vol" + i;
		getElem(btnStr).src = "Images/lcd/volume/" + btnStr + ".png";
	}
	for (i=(gVolume+1); i<=6; i++) {
		btnStr = "vol" + i;
		getElem(btnStr).src = "Images/blank.png";
	}
	
	// Update the volume popup menu
	if (gVolume>0) {
		getElem("volumePopup").selectedIndex=(gVolume-1);
	}
	
	//
	// Mute
	//
	if (gIsMuted) {
		getElem("muteCheck").checked = true;
		getElem("bell").src = "Images/blank.png";
	} else {
		getElem("muteCheck").checked = false;
		getElem("bell").src = "Images/lcd/bell.png";
	}
}


/**
 * Updates global variables, etc., based on the contents of whatever widgets
 * are currently shown in the prefs.
 */
function updateStateFromGUI()
{
	var elem = null;
	
    gShowTextMessage = getElem("showTextMessageCheck").checked;
    gTextMessage = getElem("alarmMessage").value;
    gUseGrowl = getElem("useGrowlCheck").checked;
    gAlarmSnoozeLength = parseIntFixed(getElem("alarmSnoozeDelay").value);
    gMuteBetweenEnabled = getElem("muteBetweenCheck").checked;
    gMuteBetweenStart = (parseTimeInputElem("muteStartTime")).getTime();
    gMuteBetweenEnd = (parseTimeInputElem("muteEndTime")).getTime();
    
	// Preset vs. custom interval
    gOtherUseCustomInterval = getElem("intervalCustom").checked;
	
	// Get the time interval
	if (gChimeType == "clockChimes") {
		elem = getElem("chimesMenu");
		gClockChimeInterval = parseInt(elem.options[elem.selectedIndex].value);
	}
    
    else if (gChimeType == "alarm") {
        gAlarmTime = (parseTimeInputElem("alarmTime")).getTime();
        gAlarmRepeatUntilDismissed = getElem("repeatUntilDismissedCheck").checked;
        ///gInterval = 1;  // This can be anything but 0 when clock alarms are being used.
    }
    
    else if (gChimeType == "interval") {
		var userTimeHours = parseIntFixed(getElem("customTimeHours").value);
		var userTimeMins = parseIntFixed(getElem("customTimeMin").value);
		if (userTimeHours || userTimeMins) {
			var mins = 0;
			if (userTimeMins) {
				if (userTimeMins>0) {
					mins += userTimeMins;
				}
			}
			if (userTimeHours) {
				if (userTimeHours>0) {
				mins += userTimeHours*60;
				}
			}
			gOtherCustomInterval = mins;
		} else {
			gOtherCustomInterval = 1;
		}
        
        // Other time interval menu
        elem = getElem("intervalMenu");
        gOtherPresetInterval = parseInt(elem.options[elem.selectedIndex].value);
	}
    
	
	// Volume
	gVolume = parseInt(getElem("volumePopup").value);
	
	// Mute
	gIsMuted = getElem("muteCheck").checked;
	
	// Chime type
	gChimeType = getElem("chimesChoiceMenu").value;
}



// -----------------------------------------------------------------------------
// Chime scheduling stuff
// -----------------------------------------------------------------------------


/**
 * Based on the global settings, returns a Date object with the exact time
 * of the next chime. (Exact within one second, currently.)
 */
function timeOfNextChime()
{
	var now = new Date();
	var hours = now.getHours();
	var minutes = now.getMinutes();
	
	var extraMinutes = 0;
	var newDate = null;
	
	// A Date object representing the current time, but with seconds and ms zeroed out
	var cleanDate = new Date((now.getYear()+1900), now.getMonth(), now.getDate(), hours, minutes, 0);
	
    //
    // OS9 style chimes
    //
	if (gChimeType == "os9") {
        // If no chime is selected for any of the intervals, return null so that
        // we don't create an infinite loop in the scheduler.
        var i;
        var noOS9ChimeIsSet = true;
        for (i=0; i<4; i++) {
            if (gOS9Chimes[i]) {
                noOS9ChimeIsSet = false;
                break;
            }
        }
        if (noOS9ChimeIsSet) {
            return null;
        }
        
		// Need to figure out the right pattern for this, obviously...
		if (minutes < 15) {
			if (gOS9Chimes[1]) {
				extraMinutes = 15-minutes;
			} else if (gOS9Chimes[2]) {
				extraMinutes = 30-minutes;
			} else if (gOS9Chimes[3]) {
				extraMinutes = 45-minutes;
			} else if (gOS9Chimes[0]) {
				extraMinutes = 60-minutes;
			}
		}
		else if (minutes < 30) {
			if (gOS9Chimes[2]) {
				extraMinutes = 30-minutes;
			} else if (gOS9Chimes[3]) {
				extraMinutes = 45-minutes;
			} else if (gOS9Chimes[0]) {
				extraMinutes = 60-minutes;
			} else if (gOS9Chimes[1]) {
				extraMinutes = 75-minutes;
			}
		}
		else if (minutes < 45) {
			if (gOS9Chimes[3]) {
				extraMinutes = 45-minutes;
			} else if (gOS9Chimes[0]) {
				extraMinutes = 60-minutes;
			} else if (gOS9Chimes[1]) {
				extraMinutes = 75-minutes;
			} else if (gOS9Chimes[2]) {
				extraMinutes = 90-minutes;
			}
		}
		else if (minutes < 60) {
			if (gOS9Chimes[0]) {
				extraMinutes = 60-minutes;
			} else if (gOS9Chimes[1]) {
				extraMinutes = 75-minutes;
			} else if (gOS9Chimes[2]) {
				extraMinutes = 90-minutes;
			} else if (gOS9Chimes[3]) {
				extraMinutes = 105-minutes;
			}
		}
		
		// Calculate the new Date object for the projected time
		newDate = new Date(cleanDate.getTime() + (kMsPerMinute * extraMinutes));
		return newDate;
	}
	
    //
    // "Other" chimes
    //
	else if (gChimeType == "interval") {
        if (gOtherUseCustomInterval) {
            if (!gOtherCustomInterval) {
                gOtherCustomInterval = 1;   // Set a default greater than 0. This may not be the best place for this.
            }
            newDate = new Date(now.getTime() + (kMsPerMinute * gOtherCustomInterval));
        } else {
            if (!gOtherPresetInterval) {
                gOtherPresetInterval = 1;   // Set a default greater than 0. This may not be the best place for this.
            }
            newDate = new Date(now.getTime() + (kMsPerMinute * gOtherPresetInterval));
        }
		return newDate;
	}
	
    //
    // Clock chimes
    //
	else if (gChimeType == "clockChimes") {
		extraMinutes = gClockChimeInterval - (cleanDate.getMinutes() % gClockChimeInterval);
		newDate = new Date(cleanDate.getTime() + (kMsPerMinute * extraMinutes));
		return newDate;
	}
    
    //
    // Alarm clock
    //
    else if (gChimeType == "alarm") {
        var alarmDate = null;
        if (gAlarmSnoozeOn) {
            alarmDate = new Date(gAlarmSnoozeStartTime.getTime() + (kMsPerMinute * gAlarmSnoozeLength));
        } else {
            alarmDate = new Date(gAlarmTime);
            alarmDate.setHours(alarmDate.getUTCHours());
            alarmDate.setMinutes(alarmDate.getUTCMinutes());
            alarmDate.setYear(now.getYear()+1900);
            alarmDate.setMonth(now.getUTCMonth());
            alarmDate.setDate(now.getDate());
            if (alarmDate.getTime() < now.getTime()) {
                alarmDate.setDate(now.getDate()+1);
            }
        }
        
        return alarmDate;
    }
	
	return null;
}



var gPlayChimeTimeoutID = null;
var gChimeTime = null;  // This is necessary when checking to see if the system was asleep
/**
 * Makes sure that chimes are scheduled to play as per the user's preferences.
 * Should be called when the user dismisses the preferences, or when the widget
 * is launched.
 * Also called by playChimeAndReschedule().
 */
function startChimes()
{
    var now = new Date();
    var chimeTime = timeOfNextChime();
    
    //
    // If a future chime is arranged, calculate the correct delay and set a
    // timeout for playing the chime.
    //
    if (chimeTime) {
        //
        // If there is already a chime scheduled for some reason, cancel it
        //
        if (gPlayChimeTimeoutID) {
			stopChimes();
		}
        
        //
        // Schedule the next chime
        //
        var delay = chimeTime.getTime() - now.getTime();
        gPlayChimeTimeoutID = setTimeout("playChimeAndReschedule()", delay);
    }
    gChimeTime = chimeTime;
    
    //
    // Update the front display to reflect the time of the next chime
    //
    setAlertTime(chimeTime);
}


/**
 * Stops any chimes that have been scheduled to go off in the future. This
 * should be called when the user opens the preferences, or when the chimes
 * need to be reset for some reason (such as when waking up from sleep).
 */
function stopChimes()
{
    clearTimeout(gPlayChimeTimeoutID);
    gPlayChimeTimeoutID = null;
    setAlertTime(null);
}


/**
 * Plays the chime, then calls startChime() to reschedule itself. The chime
 * won't play if the system was asleep during the originally scheduled time, in
 * which case the next chime will just be scheduled.
 */
function playChimeAndReschedule()
{
    //
    // Check whether or not the system was asleep. If not, play the chime. If so,
    // reset and reschedule.
    //
    var now = new Date();
    var secondsDelta = (now.getTime() - gChimeTime.getTime()) / 1000;
    var wasAsleep = false;
    if (secondsDelta > 20) {
        wasAsleep = true;
    }
    if (!wasAsleep) {
        playChime();
    } else {
        stopChimes();
    }
    
    //
    // Schedule the next chime
    //
    startChimes();
}



var gSleepTimeTest = null;
/**
 * Handles repeating events.  This should be called every 10 seconds or so, as
 * arranged by setup().
 * Currently, the main point of this function is to test the system to see if it
 * was asleep, and reschedule chimes if necessary.
 */
function periodicStuff()
{
	//
	// Check for sleep
	//
	var now = new Date();
	var wasAsleep = false;
	//
	// Detect whether or not the computer has woken from sleep, and reset
	// the display if necessary
	//
	if (gSleepTimeTest) {
		var minutesDelta = (now.getTime() - gSleepTimeTest.getTime()) / kMsPerMinute;
		if (minutesDelta > 1) {
			wasAsleep = true;
		}
	}
	gSleepTimeTest = now;
	// If we were asleep during the chime timeout, restart the alerts
	if (wasAsleep) {
        var secondsDelta = (now.getTime() - gChimeTime.getTime()) / 1000;
        if (secondsDelta > 10) {
            stopChimes();
            startChimes();
        }
	}
	
	//
	// Schedule the next chime
	//
    /*
	if ((!gPlayChimeTimeoutID) && (gChimeType != "off"))  {
        alert("periodicStuff(): calling startChimes()");
		startChimes();  // *** Is this really a good idea?
	}
    */
}




/**
 * Activates the snooze feature if a timed alarm is active.
 */
function startSnooze()
{
    //
    // Only fiddle with the snooze when a timed alarm is actually active
    //
    if (gAlarmOn) {
        gAlarmSnoozeOn = true;
        
        // Turn off the current alarm sound
        stopPlayer();
        stopChimes();
        
        // Reset the snooze timer
        gAlarmSnoozeStartTime = new Date();
        
        // Restart stuff
        startChimes();
        
        updateGUI();
    }
    
    closeBalloon();
}


/**
 * Indicates to the scheduler that an alarm is currently playing.
 */
function startAlarm()
{
    // Not much here yet...
    gAlarmSnoozeOn = false;
    gAlarmOn = true;
}


/**
 * Stops the alarm that is currently playing and schedules the next alarm.
 */
function stopAlarm()
{
    gAlarmSnoozeOn = false;
    gAlarmOn = false;
    gAlarmSnoozeStartTime = null;
    gAlarmStartTime = null;
    
    // Stop the current alerts
    stopPlayer();
    stopChimes();
    
    // Restart stuff
    startChimes();
    
    closeBalloon();
}




/**
 * Based on global settings, plays the chime.  This should generally be called
 * via playChimeAndReschedule().
 */
function playChime()
{
	if (gChimeType == "off") {
        //
        // If chimes are turned off, we shouldn't even reach this bit of code,
        // but just in case...
        //
        gPlayChimeTimeoutID = null;
        return;
    }
    
	var now = new Date();
	var soundPath = null;
    
    //
    // Get the path to the sound that we need to play
    //
    if (gChimeType != "os9") {
        //
        // Most chime types use the same sound choice
        //
        soundPath = gPathToSound;
    } else {
        //
        // In the case of OS9 chimes, we need to figure out which of the four
        // possible sounds needs to be played.
        //
        switch (now.getMinutes()) {
            case 0:
                soundPath = gOS9Chimes[0];
                break;
            case 15:
                soundPath = gOS9Chimes[1];
                break;
            case 30:
                soundPath = gOS9Chimes[2];
                break;
            case 45:
                soundPath = gOS9Chimes[3];
                break;
        }
    }
    
    //
    // Do we need to play the sound repeatedly?
    //
    var repeatCount = 1;
    if ((now.getMinutes() === 0) && 
        ((gOS9StrikeHour && (gChimeType == "os9")) ||
         (gClockChimeStrikeHour && (gChimeType == "clockChimes"))))
    {
        //
        // If clock-synchronized chimes are being used (either OS9 Style or
        // Clock Chimes, at present) AND it is the top of the hour AND the user
        // has selected "strike the hour", figure out how many times the chime
        // needs to be repeated.
        //
        if (g24HourClock) {
            repeatCount = now.getHours();
        } else {
            repeatCount = get12HourTime(now.getHours())[0];
        }
    } else if (gChimeType == "alarm") {
        // *** Kludge to allow the sound to repeat for the alarm
        if (gAlarmRepeatUntilDismissed) {
            repeatCount = 2;
        }
    }
    
    //
    // Since the alarm clock has special requirements beyond simply sounding the
    // chime (e.g. repeating until turned off, the snooze feature, etc.), some
    // additional initialization is required when starting the chime.
    //
    if ((gChimeType == "alarm") && gAlarmRepeatUntilDismissed) {
        // We shouldn't call this if "Repeat until dismissed" is unchecked,
        // because none of the special alarm features apply in that case.
        // (Maybe the Snooze and Off buttons shouldn't be shown, then. Still
        // need to decide about this...
        startAlarm();
    }
    
    //
    // Show a text notification, if the user has this set up
    //
    if (gShowTextMessage) {
        showAlertMessage(false);
    }
    
    //
    // Play the sound, if we have the path and mute isn't turned on
    //
    if (soundPath && !isMuted()) {
        playSound(soundPath, repeatCount, 250);
    }
	
	//
	// Clear the global, since we're done with it
    //
	gPlayChimeTimeoutID = null;
}


/**
 * Returns true if the sound should be muted, false otherwise. (Only pays
 * attention to the time range mute at the moment.)
 */
function isMuted()
{
    var now = new Date();
    var then = now;
    var muteOn = false;
    
    if (gMuteBetweenEnabled && (gMuteBetweenStart != gMuteBetweenEnd)) {
        then.setUTCHours(now.getHours());
        then.setUTCMinutes(now.getMinutes());
        then.setUTCSeconds(0);
        var muteStartDate = new Date(gMuteBetweenStart);
        muteStartDate.setUTCFullYear(now.getUTCFullYear());
        muteStartDate.setUTCMonth(now.getUTCMonth());
        muteStartDate.setUTCDate(now.getUTCDate());
        var muteEndDate = new Date(gMuteBetweenEnd);
        muteEndDate.setUTCFullYear(now.getUTCFullYear());
        muteEndDate.setUTCMonth(now.getUTCMonth());
        muteEndDate.setUTCDate(now.getUTCDate());
        if (gMuteBetweenStart > gMuteBetweenEnd) {  // If the time overlaps midnight...
            muteEndDate.setUTCDate(now.getUTCDate() + 1);
            if ((then.getTime() < muteEndDate.getTime()) && 
                (then.getTime() < muteStartDate.getTime()))
            {
                then.setUTCDate(now.getUTCDate() + 1);
            }
        }
        
        if ((then.getTime() >= muteStartDate.getTime()) && (then.getTime() <= muteEndDate.getTime())) {
            muteOn = true;
        }
    }
    
    return muteOn;
}


/**
 * Display a message using Growl or the word balloon.
 */
function showAlertMessage(isSticky)
{
    if (gUseGrowl && gGrowlExists) {
        showGrowlAlert("ProdMe Alert", gTextMessage, isSticky);
    }
    setBalloonText("Alert", gTextMessage, true);
    showBalloon(false);
    
    //
    // Bring Dashboard to the front in order to show the message if Growl isn't being used.
    //
    if (!gUseGrowl || !gGrowlExists) {
        widget.openApplication("com.apple.dashboardlauncher");
    }
}


/**
 * Returns true if the front face of the widget is visible, false otherwise.
 * Used in the balloon code to determine the correct size of the widget.
 */
function frontIsVisible()
{
    if (getElem("front").style.display != "none") {
        return true;
    } else {
        return false;
    }
}


// -----------------------------------------------------------------------------
// Hiding and showing preferences (Apple functions)
// -----------------------------------------------------------------------------

/**
 * showPrefs() is called when the preferences flipper is clicked upon.  It
 * freezes the front of the widget, hides the front div, unhides the back div, 
 * and then flips the widget over.
 */
function showPrefs()
{
    populateSoundsMenus();
    
	var front = getElem("front");
	var back = getElem("back");
	
	if (window.widget) {
		//window.resizeTo(209, 280);
		window.resizeTo(gCurrentBackWidth, 280);
        back.style.width = gCurrentBackWidth + "px";
		setChimeType(gChimeType);
		// Freezes the widget so that you can change it without the user noticing
		widget.prepareForTransition("ToBack");
		
	}
	
	front.style.display="none";		// hide the front
	back.style.display="block";		// show the back
	widget.setCloseBoxOffset(20, 20);
	
	if (window.widget) {
		// ...and flip the widget over
		setTimeout('widget.performTransition();', 0);
	}
	
	// Clean up the front side - hide the circle behind the info button
	getElem("fliprollie").style.display = 'none';
    
    stopChimes();
}


/**
 * hidePrefs() is called by the done button on the back side of the widget.  It
 * performs the opposite transition as showPrefs() does.
 */
function hidePrefs()
{
	// We will change the timer at this point
	///if (gLastTime === null) {
	if (true) {
        closeBalloon();
		updateStateFromGUI();
		updateGUI();
		startChimes();
	}
	
	var front = getElem("front");
	var back = getElem("back");
	var about = getElem("about");
	
	if (window.widget) {
		// Freezes the widget and prepares it for the flip back to the front
		widget.prepareForTransition("ToFront");
	}
	
	//window.resizeTo(209, 158);
	window.resizeTo(gCurrentFrontWidth, gCurrentFrontHeight);
	widget.setCloseBoxOffset(26, 38);
	back.style.display="none";			// hide the back
	about.style.display="none";			// hide the back
	front.style.display="block";		// show the front
	
	if (window.widget) {
		// and flip the widget back to the front
		setTimeout ('widget.performTransition();', 0);
	}
}


if (window.widget)
{
	widget.onremove = onremove;
	//widget.onhide = onhide;
	//widget.onshow = onShow;
}

function onShow()
{
	//alert("onShow()");
}



