/*
================================================================================

ProdMe dashboard widget

Author: Jim Carlson (jacF93@hampshire.edu, http://ucsu.colorado.edu/~carlson)
  Date: 1/25/06

File: balloon.js
Description: Balloon-help-like message bubbles

================================================================================
*/

// -----------------------------------------------------------------------------
// Globals
// -----------------------------------------------------------------------------

var gWidgetNativeWidth = 209;
var gWidgetNativeHeight = 158;
var gInfoWindowVisible = true;

// Declarations to make jslint more useful
/*global alert, document, clearTimeout, getElem, setTimeout, setInterval, clearInterval, widget, window */


// -----------------------------------------------------------------------------
// Everything else
// -----------------------------------------------------------------------------


var gFontLineHeight = 15;
function calcStringSize(text, elemWidth)
{
	var testDiv = getElem("stringWidthTest");
	var trueHeight = false;
	if (parseInt(elemWidth)) {
		testDiv.style.width = parseInt(elemWidth) + "px";
		trueHeight = true;
	} else {
		testDiv.style.width = "100px";
	}
	testDiv.innerHTML = text;
	
	var lineHeight = gFontLineHeight;
	var lineCount = (parseInt(testDiv.clientHeight/lineHeight)) + 1;
	lineCount = (lineCount >= 1) ? lineCount : 1;
	var width = testDiv.clientWidth * lineCount;
	var height = testDiv.clientHeight;
	
	var size;
	if (trueHeight) {
		width = testDiv.clientWidth;
		size = [width, height];
	} else {
		size = [width, lineHeight];
	}
	
	return size;
}



function calcBalloonTextSize(text)
{
	var size = [0, 0];
	
	var pixels = calcStringSize(text);
	var lineHeight = gFontLineHeight;
	var textWidth = pixels[0];
	var lineCount = 1;
	// Iterative...  I'm sure there's a better way, but I'm lazy.
	for (var i=1; i<100; i++) {
		var rem = (textWidth/i) % 1;
		var width = parseInt(textWidth/i);
		lineCount = i;
		if (rem>0) { lineCount++; }
		var height = lineCount * lineHeight;
		if ((width/height)<1.7) {
			size = [width, height];
			break;
		}
	}
    
	return size;
}



var gStickyBalloon = false;
function setBalloonText(title, text, showCloseBox)
{
	var size = calcBalloonTextSize(text);
	var width = size[0] + 100;
	
	var newSize = calcStringSize(text, size[0]);
	var height = newSize[1] + 60;
	setBalloonSize(width, height);
	
	getElem("balloonMessage").innerHTML = text;
	getElem("balloonTitle").innerHTML = title;
	
	if (showCloseBox || gStickyBalloon) {
		getElem("balloonCloseBox").style.display = "block";
		gStickyBalloon = true;
	} else {
		getElem("balloonCloseBox").style.display = "none";
	}
}


function handleBalloonCloseClick()
{
	gStickyBalloon = false;
	closeBalloon();
	getElem("balloonHelp").style.opacity = "0.0";
}


// This should do the fade animation, but I'm lazy and don't feel like
// rewriting Apple's code.
function closeBalloon()
{
	getElem("balloonHelp").style.opacity = "0.0";
	gInfoWindowVisible = false;
    
    // Resize the front face of the widget, if it's visible
    if (frontIsVisible()) {
        window.resizeTo(gWidgetNativeWidth, gWidgetNativeHeight);
    }
    gCurrentFrontWidth = gWidgetNativeWidth;
    gCurrentFrontHeight = gWidgetNativeHeight;
}


function showBalloon(sticky)
{
	var width = gBalloonTotalWidth + gWidgetNativeWidth;
	var height = (gBalloonTotalHeight < gWidgetNativeHeight) ? gWidgetNativeHeight : gBalloonTotalHeight;
    
    // Resize the front face of the widget, if it's visible
    if (frontIsVisible()) {
        window.resizeTo(width, height);
    }
    gCurrentFrontWidth = width;
    gCurrentFrontHeight = height;
    
	// Should fade in
	getElem("balloonHelp").style.opacity = "1.0";
	if (sticky) { gStickyBalloon = true; }
	gInfoWindowVisible = true;
}


var gBalloonTotalWidth = 100;
var gBalloonTotalHeight = 100;
function setBalloonSize(width, height)
{
	gBalloonTotalWidth = width;
	gBalloonTotalHeight = height;
	
	var horzDiff = width - (68 + 34);
	var vertDiff = height - (61 + 35);
	horzDiff = (horzDiff>0) ? horzDiff : 1;
	vertDiff = (vertDiff>0) ? vertDiff : 1;
	
	getElem("balloonHelp").style.width = ((horzDiff + (68 + 34)) + "px");
	getElem("balloonHelp").style.height = ((vertDiff + (61 + 35)) + "px");
}




