/*
================================================================================

ProdMe dashboard widget

Author: Jim Carlson (jacF93@hampshire.edu, http://ucsu.colorado.edu/~carlson)
  Date: 5/17/06

File: utilities.js
Description: Non-project-specific utility functions and data structures.

================================================================================
*/

// -----------------------------------------------------------------------------
// Globals
// -----------------------------------------------------------------------------



// Declarations to make jslint more useful
/*global alert, document, clearTimeout, setTimeout, setInterval, clearInterval, widget, window */


// -----------------------------------------------------------------------------
// Form element generation
// -----------------------------------------------------------------------------

function makeCheckboxElem(id, handlerName, label, title)
{
    var container = getElem(id);
    
    var labelClickHandlerStr = "getElem('" + id + "Check').checked = !(getElem('" + id + "Check').checked); " + handlerName + "(getElem('" + id + "Check'));";
    var checkboxStr = "<input id='" + id + "Check' type='checkbox' value='true' style=\"left: 0px; top: 0px; font: 10px 'Helvetica Neue';\" onchange='" + handlerName + "(this);' checked title='" + title + "'>";
    var labelStr = "<div class='prefsLabel' id='" + id + "Label' style=\"left: 16px; top: 1px;\" onClick=\"" + labelClickHandlerStr + "\" title='" + title + "'>" + label + "</div>";
    var htmlStr = checkboxStr + labelStr;
    
    container.innerHTML = htmlStr;
}


// Returns a Date object
function parseTimeInputElem(id)
{
    var hours = 0;
    var minutes = 0;
    var seconds = 0;
    var is24hour = true;
    var isPM = false;
    
    if (getElem(id + "AMPMMenu")) {
        is24hour = false;
        if (getElem(id + "AMPMMenu").value == "pm") {
            isPM = true;
        }
    }
    if (getElem(id + "Hours")) {
        hours = parseIntFixed(getElem(id + "Hours").value);
        if (!is24hour) {
            if (!isPM && (hours == 12)) {
                hours = 0;      // 12 AM
            } else if (isPM && (hours < 12)) {
                hours += 12;    // This crap gets really annoying after a while...
            }
        }
    }
    if (getElem(id + "Minutes")) {
        minutes = parseIntFixed(getElem(id + "Minutes").value);
    }
    if (getElem(id + "Seconds")) {
        seconds = parseIntFixed(getElem(id + "Seconds").value);
    }
    
    var ms = (((60 * hours) + minutes) * kMsPerMinute) + (1000 * seconds);
    var theDate = new Date(ms);
    //alert("theDate: " + theDate.toUTCString());
    return theDate;
}

// theDate must be a Date object
function fillTimeInputElem(id, theDate)
{
    //alert("fillTimeInputElem(): id = " + id + ", theDate = " + theDate.toLocaleString());
    alert("fillTimeInputElem(): id = " + id + ", theDate = " + theDate);
    var is24hour = true;
    var isPM = false;
    var amPmMenu = getElem(id + "AMPMMenu");
    
    if (amPmMenu) {
        is24hour = false;
    }
    if (getElem(id + "Hours")) {
        var hoursElem = getElem(id + "Hours");
        var hours = theDate.getUTCHours();
        if (!is24hour) {
            if (hours == 0) {
                hoursElem.value = 12;
                amPmMenu.value = "am";
            } else if (hours >= 12) {
                amPmMenu.value = "pm";
                if (hours > 12) {
                    hoursElem.value = hours - 12;
                } else {
                    hoursElem.value = 12;
                }
            } else {
                hoursElem.value = hours;
                amPmMenu.value = "am";
            }
        } else {
            hoursElem.value = hours;
        }
    }
    if (getElem(id + "Minutes")) {
        var minutes = theDate.getUTCMinutes();
        if (getElem(id + "Hours") && (minutes < 10)) {
            minutes = "0" + minutes;
        }
        getElem(id + "Minutes").value = minutes;
    }
    if (getElem(id + "Seconds")) {
        var seconds = theDate.getUTCSeconds();
        if (seconds < 10) {
            seconds = "0" + seconds;
        }
        getElem(id + "Seconds").value = seconds;
    }
}


function makeTimeInputElem(id, handlerName, showHours, showMinutes, showSeconds, showAmPm, is24Hour)
{
    var leftLoc = 0;
    var hoursStr   = "";
    var sep1Str    = "";
    var minutesStr = "";
    var sep2Str    = "";
    var secondsStr = "";
    var amPmStr    = "";
    
    if (showHours) {
        hoursStr = "<input class='prefsInput' id='" + id + "Hours' style='left: " + leftLoc + "px; top: 0px; text-align: right;' type='text' value='12' size='2' maxlength='2' title='Hours' onKeyDown='handleNumberKeyPress(this);' onBlur='handleNumberOnBlur(this, 12, 1, 12, false); " + handlerName + "(this);'>\n";
        leftLoc += 23;
        if (showMinutes || showSeconds) {
            sep1Str = "<div class='prefsLabel' style='left: " + leftLoc + "px; top: 2px;'>:</div>\n";
            leftLoc += 6;
        }
    }
    
    if (showMinutes) {
        var maxMins = (showHours) ? 59 : 99;    // *** Is this really a good idea?
        minutesStr = "<input class='prefsInput' id='" + id + "Minutes' style='left: " + leftLoc + "px; top: 0px; text-align: right;' type='text' value='00' size='2' maxlength='2' title='Minutes' onKeyDown='handleNumberKeyPress(this);' onBlur='handleNumberOnBlur(this, 0, 0, " + maxMins + ", true); " + handlerName + "(this);'>\n";
        leftLoc += 23;
        if (showSeconds) {
            sep2Str = "<div class='prefsLabel' style='left: " + leftLoc + "px; top: 2px;'>:</div>\n";
            leftLoc += 6;
        }
    }
    
    if (showSeconds) {
        secondsStr = "<input class='prefsInput' id='" + id + "Seconds' style='left: " + leftLoc + "px; top: 0px; text-align: right;' type='text' value='00' size='2' maxlength='2' title='Seconds' onKeyDown='handleNumberKeyPress(this);' onBlur='handleNumberOnBlur(this, 0, 0, 59, true); " + handlerName + "(this);'>\n";
        leftLoc += 23;
    }
    
    if (showAmPm) {
        leftLoc += 2;
        amPmStr = "<select class='prefsMenu' id='" + id + "AMPMMenu' style='left: " + leftLoc + "px; top: 2px;' onChange='" + handlerName + "(this);'><option value='am'>AM</option><option value='pm'>PM</option></select>\n";
    }
    
    var htmlStr = hoursStr + sep1Str + minutesStr + sep2Str + secondsStr + amPmStr;
    
    getElem(id).innerHTML = htmlStr;
}


// -----------------------------------------------------------------------------
// Everything else
// -----------------------------------------------------------------------------

/**
 * Workaround for a nasty bug in Apple's implementation of parseInt().
 */
function parseIntFixed(val)
{
    return Math.round(parseFloat(val));
}

/**
 * Checks whether or not the system clock is set to 24 hour display.
 */
function is24HourClock()
{
	var elevensTime = new Date(2000, 0, 1, 23, 11, 11, 0);
	var localTimeString = elevensTime.toLocaleString();
	
	var count11 = 0;
	// Should just look for 3 instances of the number 11
	var idx = -1;
	for (var i=0; i<3; i++) {
		idx = localTimeString.search("11");
		if (idx>-1) {
			count11++;
			localTimeString = localTimeString.substr(idx+2);
		}
		else {
			break;
		}
	}
	
	var returnVal = (count11==3) ? false : true;
	return returnVal;
}

/**
 * For now, just returns the array [<hours>, <am|pm>]
 */
function get12HourTime(hours)
{
	if ((hours<0) || (hours>23)) {
		return null;
	}
	
	var amPm = "am";
	if (hours == 12) { amPm = "pm"; }
	if (hours > 12) {
		hours -= 12;
		amPm = "pm";
	} else if (hours === 0) {
		hours = 12;
	}
	
	return [hours, amPm];
}


/**
 * Get the current username (swiped directly from YOLK widget).
 */
function getUser() 
{
	var currentUser = "";
	var trimRegExp = /^[ \t]+|[ \t]+$/;
	var trimMoreRegExp = /^[ \t\n\r]+|[ \t\n\r]+$/;
	
	if (window.widget) {
		currentUser = widget.system("/usr/bin/whoami", null).outputString;
		currentUser = currentUser.replace(trimRegExp, "");
		currentUser = currentUser.replace(trimMoreRegExp, "");
	}
	
	return currentUser;
}


var getWidgetRoot_WidgetRoot = null;
/**
 * Returns the path of the root directory of the widget
 */
function getWidgetRoot()
{
	//if (!getWidgetRoot_WidgetRoot) {
	if (true) {
		var loc = document.location.href;
		var idx = loc.indexOf('file:///');
		if (idx == -1) {
			idx = loc.indexOf('file:/');
			if (idx === 0) {
				loc = "file:///" + loc.substring(6);
			}
		}
		
		getWidgetRoot_WidgetRoot = loc.substring(0, loc.lastIndexOf('/') + 1);
	}
	
	return getWidgetRoot_WidgetRoot;
}


var getElem_Cache = new Array();
/**
 * Caching mechanism to replace document.getElementById() for reasons of 
 * efficiency.
 */
function getElem(idStr) {
	var elem = getElem_Cache[idStr];
	if (!elem) {
		elem = document.getElementById(idStr);
		getElem_Cache[idStr] = elem;
	}
	return elem;
}



// -----------------------------------------------------------------------------
// Growl stuff
// -----------------------------------------------------------------------------

/**
 * Shows a Growl notification.
 * @param title The title of the notification.
 * @param desc The text in the body of the notification.
 * @param isSticky If true, the notification will remain visible until the user manually dismisses it.
 */
function showGrowlAlert(title, desc, isSticky)
{
    var pwdStr = getWidgetRoot();
	
	var imagePath = pwdStr + "/Icon.png";
	imagePath = imagePath.replace(/ /g, "\\ ");
	var cmdStr = "/usr/bin/osascript Scripts/growl/alert.scpt " + imagePath;
	cmdStr += " \"" + title + "\" \"" + desc + "\"" + " " + isSticky;
	//alert("showGrowlAlert(): cmdStr = " + cmdStr);
	var obj = widget.system(cmdStr, null);
}

/**
 * Returns true if Growl is available, false otherwise.
 */
function registerWithGrowl()
{
    //
    // Attempt to register with Growl.  Also determines whether or not Growl
    // is available.
    //
	var output = widget.system("/usr/bin/osascript Scripts/growl/register.scpt", null).outputString;
    var err = output.match(/error/);
    if (!err) {
        return true;
    } else {
        return false;
    }
}




// -----------------------------------------------------------------------------
// Preferences
// -----------------------------------------------------------------------------

/**
 * Create a unique preference key for this instance of the widget (taken from
 * Apple's Weather widget)
 * @param key the name of the preference setting
 * @return a string with key prefixed by the identifier number and a dash.
 */
function createkey(key)
{
	return widget.identifier + "-" + key;
}

var kFakeNull = "RtdRGaS3WxOxiOg820v2n";	// Generated at random from Apple's password utility
function storePref(value, key, leaveDefaults)
{
	if (!value) { value = kFakeNull; }
	if (window.widget) {
		widget.setPreferenceForKey(value, createkey(key));
		
		if (!leaveDefaults) {
			widget.setPreferenceForKey(value, key);
		}
	}
}


function clearPref(key)
{
	if (window.widget) {
		widget.setPreferenceForKey(null, createkey(key));
	}
}


function retrievePref(key, defaultVal)
{
    if ((defaultVal == undefined) && (defaultVal !== 0)) {
        //alert("retrievePref(): " + key + ", defaultVal = " + defaultVal);
        defaultVal = null;
    }
	if (window.widget) {
		var value = widget.preferenceForKey(createkey(key));
		if (!value) {
			value = widget.preferenceForKey(key);
		}
		if (!value) {
			value = defaultVal;
		} else if (value == kFakeNull) {
			return null;
		}
		return value;
	}
	else {
		return null;
	}
}



// -----------------------------------------------------------------------------
// Image button stuff
//
// These functions replicate some of the functionality of Apple's 
// createDefaultButton() routine, because the buttons generated by
// createDefaultButton() look like ass.  The functions below work using two
// pre-rendered images per button, one image for the normal button state, and
// the other for the pressed state.
// -----------------------------------------------------------------------------

var gLastBtnPressed = null; // Stores the button that was pressed

/**
 * Sets up the event handlers for the button.  The HTML for the button should be
 * formatted like this:   &lt;img id="genericButton" /&gt;
 * @param id the id of the button element
 * @param upImage the relative path of the normal state image
 * @param pressedImage the relative path of the depressed state image
 * @param the action to be performed when the button is clicked
 */
function setupImageButton(id, upImage, pressedImage, action)
{
	var btn = getElem(id);
	if (btn) {
		btn.setAttribute("onMouseDown", "buttonMouseDownHandler(event);");
		btn.setAttribute("onMouseOut", "buttonMouseOutHandler(event);");
		btn.setAttribute("onMouseUp", "buttonMouseUpHandler(event);");
		btn.setAttribute("onMouseOver", "buttonMouseOverHandler(event);");
		btn.setAttribute("onClick", "buttonOnClickHandler(event);");
		btn.dimsrc = pressedImage;
		btn.upsrc = upImage;
		btn.src = upImage;
		btn.clickAction = action;
		btn.isEnabled = true;
	}
}

function enableButton(id)
{
	getElem(id).isEnabled = true;
}

function disableButton(id)
{
	getElem(id).isEnabled = false;
}


/**
 * Used by setupImageButton.  Do not call directly.
 */
function buttonMouseDownHandler(event)
{
	var btn = event.srcElement;
	if (btn.isEnabled) {
		btn.src = btn.dimsrc;
		gLastBtnPressed = btn;
	}
}

/**
 * Used by setupImageButton.  Do not call directly.
 */
function buttonMouseOutHandler(event)
{
	var btn = event.srcElement;
	if (btn.isEnabled) {
		if ((event.button==1) && (btn.upsrc) && (gLastBtnPressed == btn)) {
			btn.src = btn.upsrc;
		}
	}
}

/**
 * Used by setupImageButton.  Do not call directly.
 */
function buttonMouseOverHandler(event)
{
	var btn = event.srcElement;
	if (btn.isEnabled) {
		if ((event.button==1) && (btn.dimsrc) && (gLastBtnPressed == btn)) {
			btn.src = btn.dimsrc;
		}
	}
}

/**
 * Used by setupImageButton.  Do not call directly.
 */
function buttonMouseUpHandler(event)
{
	var btn = event.srcElement;
	if (btn.isEnabled) {
		if ((event.button==1) && (btn.upsrc) && (gLastBtnPressed == btn)) {
			btn.src = btn.upsrc;
			btn.clickAction(event);
			gLastBtnPressed = null;
		}
	}
}

/**
 * Used by setupImageButton.  Do not call directly.
 */
function buttonOnClickHandler(event)
{
	var btn = event.srcElement;
	if (btn.isEnabled) {
		if ((event.button==1) && (btn.upsrc) && (gLastBtnPressed == btn)) {
			btn.src = btn.upsrc;
			btn.clickAction(event);
			gLastBtnPressed = null;
		}
	}
}





// -----------------------------------------------------------------------------
// Hiding and showing preferences (Apple functions)
// -----------------------------------------------------------------------------


// PREFERENCE BUTTON ANIMATION (- the pref flipper fade in/out)
var flipShown = false;	// a flag used to signify if the flipper is currently shown or not.


// A structure that holds information that is needed for the animation to run.
var animation = {duration:0, 
				 starttime:0, 
				 to:1.0,
				 now:0.0,
				 from:0.0,
				 firstElement:null,
				 timer:null};


// mousemove() is the event handle assigned to the onmousemove property on the 
// front div of the widget. It is triggered whenever a mouse is moved within the
// bounds of your widget.  It prepares the preference flipper fade and then
// calls animate() to performs the animation.
function mousemove (event)
{
	// If the preferences flipper is not already showing...
	if (!flipShown) {
		// ...reset the animation timer value, in case a value was left behind
		if (animation.timer !== null) {
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13;	// set it back one frame
		
		animation.duration = 500;									// animation time, in ms
		animation.starttime = starttime;							// specify the start time
		animation.firstElement = getElem("flip");	// specify the element to fade
		animation.timer = setInterval ("animate();", 13);			// set the animation function
		animation.from = animation.now;								// beginning opacity (not ness. 0)
		animation.to = 1.0;											// final opacity
		animate();													// begin animation
		flipShown = true;											// mark the flipper as animated
	}
}


// mouseexit() is the opposite of mousemove() in that it preps the preferences
// flipper to disappear.  It adds the appropriate values to the animation data
// structure and sets the animation in motion.
function mouseexit (event)
{
	if (flipShown)
	{
		// fade in the flip widget
		if (animation.timer !== null)
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13;
		
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = getElem("flip");
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.15;		// Keep the "i" semi-transparent
		animate();
		flipShown = false;
	}
}


// animate() performs the fade animation for the preferences flipper. It uses
// the opacity CSS property to simulate a fade.
function animate()
{
	var T;
	var ease;
	var time = (new Date).getTime();
	
	T = limit_3(time-animation.starttime, 0, animation.duration);
	
	if (T >= animation.duration) {
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else {
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		//alert("ease = " + ease);
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}
	
	animation.firstElement.style.opacity = animation.now;
}


// These functions are utilities used by animate()
function limit_3 (a, b, c)
{
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease)
{
    return from + (to - from) * ease;
}


// These functions are called when the info button itself receives onmouseover
// and onmouseout events
function enterflip(event)
{
	getElem("fliprollie").style.display = 'block';
}

function exitflip(event)
{
	getElem("fliprollie").style.display = 'none';
}